@@ -1,3 +1,9 @@
+/* 
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 /*
  * DANG_BEGIN_MODULE
  *
@@ -30,7 +36,7 @@
 
 #include <features.h>
 #include <sys/time.h>
-#if __GLIBC__ > 1
+#if GLIBC_VERSION_CODE >= 2000
 #include <sys/kd.h>
 #else
 #include <linux/kd.h>
@@ -336,10 +342,9 @@ Bit8u pit_inp(ioport_t port)
   int ret = 0;
   port -= 0x40;
 
-  if (port == 2) {
-    if (config.speaker == SPKR_NATIVE) return safe_port_in_byte(0x42);
-    pit_latch(port);
-    return(pit[port].outpin);
+  if ((port == 2) && (config.speaker == SPKR_NATIVE)) {
+	/* how could we come here if we defined PORT_FAST? */
+	return safe_port_in_byte(0x42);
   }
   else if (port == 1)
     i_printf("PIT:  someone is reading the CMOS refresh time?!?");
@@ -381,6 +386,7 @@ void pit_outp(ioport_t port, Bit8u val)
   if (port == 1)
     i_printf("PORT: someone is writing the CMOS refresh time?!?");
   else if (port == 2 && config.speaker == SPKR_NATIVE) {
+    /* how could we come here if we defined PORT_FAST? */
     safe_port_out_byte(0x42, val);
     return;
   }
@@ -465,28 +471,30 @@ void pit_control_outp(ioport_t port, Bit8u val)
   switch (latch) {
     case 2:
       if (config.speaker == SPKR_NATIVE) {
+	/* hmmm.. timer 2 is in PORT_FAST mode, and there will be a
+	 * noticeable amount of time from this mode setting to the
+	 * timer freq settings... could this cause strange effects?
+	 */
         safe_port_out_byte(0x43, val);
 	break;
       }
       /* nobreak; */
     case 0:
     case 1:
-#ifdef MONOTON_MICRO_TIMING
-      pit[latch].mode        = (val >> 1) & 0x07;
-      if ((val & 4)==0) {      /* modes 0,1,4,5 */
-      /* set the time base for the counter - safety code for programs
-       * which use a non-periodical mode without reloading the counter
-       */
-       pit[latch].time.td = GETtickTIME(0);
-      }
-#endif
       if ((val & 0x30) == 0)
 	pit_latch(latch);
       else {
 	pit[latch].read_state  = (val >> 4) & 0x03;
 	pit[latch].write_state = (val >> 4) & 0x03;
-#ifndef MONOTON_MICRO_TIMING
 	pit[latch].mode        = (val >> 1) & 0x07;
+#ifdef MONOTON_MICRO_TIMING
+        pit[latch].mode        = (val >> 1) & 0x07;
+        if ((val & 4)==0) {      /* modes 0,1,4,5 */
+          /* set the time base for the counter - safety code for programs
+           * which use a non-periodical mode without reloading the counter
+           */
+          pit[latch].time.td = GETtickTIME(0);
+        }
 #endif
       }
 #if 0
@@ -604,12 +612,15 @@ void timer_int_engine(void)
 Bit8u spkr_io_read(ioport_t port) {
    if (port==0x61)  {
       if (config.speaker == SPKR_NATIVE)
+	 /* how could we come here if we defined PORT_FAST? */
          return port_safe_inb(0x61);
-      else
+      else {
 	 /* keep the connection between port 0x61 and PIT timer#2 */
+	 pit_latch(2);
          return ((*((Bit8u *)&pic_sys_time)&0x10) | /* or anything that toggles quick enough */
-		(pit_inp(0x42)? 0x20:0) |	/* outpin: 00 or 80 */
+		(pit[2].outpin? 0x20:0) |	/* outpin: 00 or 80 */
 		(port61&0xcf));
+      }
    }
    return 0xff;
 }
@@ -618,6 +629,7 @@ void spkr_io_write(ioport_t port, Bit8u value) {
    if (port==0x61) {
       switch (config.speaker) {
        case SPKR_NATIVE:
+	  /* how could we come here if we defined PORT_FAST? */
           port_safe_outb(0x61, value & 0x03);
           break;
       
@@ -635,7 +647,6 @@ void spkr_io_write(ioport_t port, Bit8u value) {
 
 void pit_init(void)
 {
-#ifdef NEW_PORT_CODE
   emu_iodev_t  io_device;
 
   /* 8254 PIT (Programmable Interval Timer) */
@@ -677,7 +688,6 @@ void pit_init(void)
   io_device.end_addr     = 0x0047;
   port_register_handler(io_device, 0);
 #endif
-#endif
 }
 
 void pit_reset(void)
