@@ -121,6 +121,33 @@ static unsigned char *recode(unsigned int *out, unsigned char *in)
           }
           return in;
         }
+        case 'M': { /* 'M' ==  as in Move, 
+                       ('P' would be nicer (Pu for PageUp) but is already used...)
+                       example: \Mh  == <HOME>   \Mu  == <PageUp>
+                                \Me  == <END>    \Md  == <PageDown>
+                                \M8  == <ArrowUp>
+                                \M4  == <ArrowLeft>  \M6  == <ArrowRight>
+                                \M2  == <ArrowDown>
+                       */
+          ch=*(in++);
+          switch(ch) {
+            case 'i': keynum = 0xe052; break;  /* KEY_INS */
+            case 'h': keynum = 0xe047; break;  /* KEY_HOME */
+            case 'u': keynum = 0xe049; break;  /* KEY_PGUP */
+            case 'c': keynum = 0xe053; break;  /* KEY_DEL */
+            case 'e': keynum = 0xe04f; break;  /* KEY_END */
+            case 'd': keynum = 0xe051; break;  /* KEY_PGDN */
+            case '8': keynum = 0xe048; break;  /* KEY_UP */
+            case '4': keynum = 0xe04b; break;  /* KEY_LEFT */
+            case '6': keynum = 0xe04d; break;  /* KEY_RIGHT */
+            case '2':
+            default:  keynum = 0xe050; break;  /* KEY_DOWN */
+          }
+          *(out++) = keynum;         /* key pressed */
+          *(out++) = keynum | 0x80;  /* key released */
+          *(out++) = 0;
+          return in;
+        }
         default:
         /* fall through */
       }
@@ -169,7 +196,6 @@ int type_in_pre_strokes()
       o=out;
       while (*o) {
         int c = (*o >>8) & 0xff;
-        if (!c) c = ' ';
         putkey((*o & 0x80)==0, (t_keysym)((*o & 0x7f) | ((*o >> 8) & 0xff00)), c);
         o++;
       }
@@ -181,20 +207,27 @@ int type_in_pre_strokes()
 
 void append_pre_strokes(unsigned char *s)
 {
-  if (config.pre_stroke) {
+  if (config.pre_stroke_mem) {
     int l1,l2;
+    int offs = 0;
     unsigned char *n;
 
-    l1 = strlen(config.pre_stroke);
+    if (config.pre_stroke) {
+      l1 = strlen(config.pre_stroke);
+    }
+    else {
+      l1 = 0;
+    }
     l2 = strlen(s);
     n = malloc(l1+l2+1);
     if (!n) return;
-    memcpy(n, config.pre_stroke, l1);
+    if (l1) memcpy(n, config.pre_stroke, l1);
     memcpy(n+l1, s, l2+1);
-    free(config.pre_stroke);
+    free(config.pre_stroke_mem);
     config.pre_stroke = n;
   }
   else {
     config.pre_stroke = strdup(s);
   }
+  config.pre_stroke_mem = config.pre_stroke;
 }
