@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -54,6 +54,7 @@
 #include "pic.h"
 #include "int.h"
 #include "vc.h"
+#include "dosemu_select.h"
 
 #ifndef max
 #define max(a,b)       ((a)>(b)? (a):(b))
@@ -342,3 +343,40 @@ void run_unix_command(char *buffer)
             WEXITSTATUS(status));
     }
 }
+
+
+/*
+ * This runs system() in a forked environment with all priviledges off
+ * while scheduling vital DOSEMU functionality in the background.
+ */
+int run_system_command(char *buffer)
+{
+    int pid, status;
+
+    if (!buffer || !buffer[0]) return 1;
+
+    /* fork child */
+    pid=fork();
+    if (pid == -1) {
+        /* failed */
+        g_printf("run_system_command(): fork() failed\n");
+        return 1;
+    }
+    else if (pid == 0) {
+	/* child */
+	int retval;
+	priv_drop(); /* we drop _all_ priviledges */
+        retval=system(buffer);	/* execute command */
+        _exit(retval);
+    }
+    else {
+	/* parent */
+        while (waitpid(pid, &status, WNOHANG) != pid) {
+            handle_signals();
+            if (iq.queued)
+                  do_queued_ioctl();
+            usleep(10000);
+        }
+        return WEXITSTATUS(status);
+    }
+}
