@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -27,7 +27,7 @@
 #include <sys/stat.h>
 #include <net/if.h>
 #include <netinet/in.h>
-#if defined(__GLIBC__) && __GLIBC__ >= 2
+#if GLIBC_VERSION_CODE >= 2000
   #include <netinet/if_ether.h>
 #else
   #include <linux/sockios.h>
