@@ -1,8 +1,14 @@
+/* 
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 #ifndef _EMU_IODEV_H
 #define _EMU_IODEV_H
 
 #include <features.h>
-#if __GLIBC__ > 1 
+#if GLIBC_VERSION_CODE >= 2000
 #include <sys/time.h>
 #else
 #include <linux/time.h>
@@ -71,12 +77,7 @@ extern void  pit_reset(void);
 
 extern void  rtc_init(void);
 extern void  rtc_reset(void);
-#ifdef NEW_CMOS
 extern void  rtc_update (void);
-#else
-extern Bit8u rtc_read(Bit32u port);
-extern void  rtc_write(Bit32u port, Bit8u byte);
-#endif
 
 /*******************************************************************
  * CMOS support                                                    *
@@ -84,52 +85,10 @@ extern void  rtc_write(Bit32u port, Bit8u byte);
 
 #include "cmos.h"
 
-#ifdef NEW_CMOS
 extern long  last_ticks;		/* for int1a */
 extern long  sys_base_ticks;
 extern long  usr_delta_ticks;
 
-#else
-
-#define CMOS_SEC	0	/* RTC time: seconds */
-#define CMOS_SECALRM	1	/* Alarm time: seconds */
-#define CMOS_MIN	2	/* RTC time: minutes */
-#define CMOS_MINALRM	3	/* Alarm time: minute */
-#define CMOS_HOUR	4	/* RTC time: hours */
-#define CMOS_HOURALRM	5	/* Alarm time: hour */
-#define CMOS_DOW	6	/* Day of Week */
-#define CMOS_DOM	7	/* Day of month */
-#define CMOS_MONTH	8	/* Month */
-#define CMOS_YEAR	9	/* Year */
-#define CMOS_STATUSA	0xA	/* Status A */
-#define CMOS_STATUSB	0xB	/* Status B */
-#define CMOS_STATUSC	0xC	/* Status C */
-#define CMOS_STATUSD	0xD	/* Status D */
-#define CMOS_DIAG	0xE	/* Diagnostic status byte */
-#define CMOS_SHUTDOWN	0xF	/* shutdown status byte */
-#define CMOS_DISKTYPE	0x10	/* floppy type byte */
-#define CMOS_RESV1	0x11
-#define CMOS_HDTYPE	0x12	/* drive type byte (4 bits per drive, 0 for see extwords) */
-#define CMOS_RESV2	0x13
-#define CMOS_EQUIPMENT	0x14	/* equipment byte */
-#define CMOS_BASEMEML	0x15	/* base memory in 1k blocks */
-#define CMOS_BASEMEMM	0x16
-#define CMOS_EXTMEML	0x17	/* extended memory in 1k blocks */
-#define CMOS_EXTMEMM	0x18
-#define CMOS_HD1EXT	0x19	/* drive C: extension byte */
-#define CMOS_HD2EXT	0x1a	/* drive D: extension byte */
-#define CMOS_RESV3	0x1b
-#define CMOS_CHKSUMM	0x2e	/* checksum over 0x10-0x20 */
-#define CMOS_CHKSUML	0x2f
-#define CMOS_PEXTMEML	0x30	/* extended memory over 1 MB found during POST */
-#define CMOS_PEXTMEMM	0x31
-#define CMOS_CENTURY	0x32
-#define CMOS_INFO	0x33
-#define CMOS_RESV4	0x34	/* 12 bytes reserved */
-
-extern void  cmos_init(void);
-extern void  cmos_reset(void);
-#endif	/* NEW_CMOS */
 
 /*******************************************************************
  * PIC support                                                     *
