@@ -1,3 +1,9 @@
+/* 
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 #ifndef VIDEO_H
 #define VIDEO_H
 
@@ -192,6 +198,9 @@ EXTERN int v_8514_base INIT(0);
 #define CHARSET_LATIN	1
 #define CHARSET_IBM	2
 #define CHARSET_FULLIBM	3
+#define CHARSET_LATIN1  4
+#define CHARSET_LATIN2  5
+
 
 /* Color set defines */
 /* #define COLOR_NORMAL	1 */
